//---------------------------------------------------------------------------

#pragma hdrstop

#include "ChartDonut.h"

#include <cstdlib>
#include <ctime>

//
#include <VCLTee.TeeGDIPlus.hpp>
//---------------------------------------------------------------------------
#pragma package(smart_init)

DonutChart::DonutChart(TChart & pChart, UnicodeString pTitle) :
		_colorVect(InitColorLst())
{
	_chart = &pChart;
	_title = pTitle;

	srand(time(NULL));
	InitChart();
}

void DonutChart::InitChart()
{
	//TChart
	_chart->ClearChart();
	_chart->Legend->Visible = false;
	_chart->Title->Caption = _title;
	_chart->View3D = false;
	_chart->Title->Visible = true;
	_chart->Border->Visible = true;
	_chart->Border->Width = 3;
	if (_chart->Parent)
	{
		//with multichart, titles are draw in wrong subchart
		_chart->Title->AdjustFrame = false;
	}
	_chart->Color = clWhite;
	ConfigMarksTipTool();
	//Serie
	_serie = new TDonutSeries(_chart);
	_chart->AddSeries(_serie);
	_serie->Marks->Tail->Visible = false;
	_serie->Marks->Callout->ArrowHead = ahSolid;
	_serie->Marks->Arrow->Visible = true;
	_serie->Marks->Callout->Arrow->Visible = true;
}

void DonutChart::AddData(int piValue, UnicodeString const & pMarkText)
{
	TColor col = _colorVect[_serie->Count() % 6];
	_serie->Add(piValue, pMarkText, col);
	ConfigMarks();
}

void DonutChart::AddRandomData(int iMax)
{
	int val = rand() % iMax + 1;
	AddData(val, "RandVal :\r\n" + IntToStr(val));
}

void DonutChart::ConfigMarks()
{
	for (int i = 0; i < _serie->XValues->Count; i++)
	{
		_serie->Marks->Item[i]->Color = _serie->ValueColor[i];
		_serie->Marks->Item[i]->Font->Color = clWhite;
	}
	MoveMarks();
}

void DonutChart::MoveMarks()
{
	static bool bRedraw = false;
	if (!bRedraw && _chart->Parent != NULL)
	{
		bRedraw = true;
		//some marks not draw on auto, need to redraw to generate them.
		_serie->Marks->Automatic->Move = false;
		_chart->Draw();
		//
		TRect rectDonuts = _serie->CircleRect;
		//parcours des marks
		TSeriesMarkPosition * mkPre = NULL, *mkAct = NULL;
		TRect rectMkPre, rectMkAct;
		int posX = 0, maxposX = 0;
		for (int iIndicePortion = 0; iIndicePortion < _serie->Count(); iIndicePortion++)
		{
			mkAct = _serie->Marks->Positions->Position[iIndicePortion];
			if (mkAct != NULL)
			{
				mkAct->Custom = true;
				if (mkPre != NULL)
				{
					if (mkPre->Bounds().Bottom + 1 + mkAct->Bounds().Height() < _chart->Height)
					{
						mkAct->LeftTop.X = posX;
						mkAct->LeftTop.Y = mkPre->Bounds().Bottom + 1;
					}
					else
					{
						posX = maxposX;
						mkAct->LeftTop.X = posX;
						mkAct->LeftTop.Y = 0;
					}
				}
				else
				{
					mkAct->LeftTop.X = 0;
					mkAct->LeftTop.Y = 0;
				}
				if (mkAct->Bounds().Right > maxposX)
				{
					maxposX = mkAct->Bounds().Right;
				}
				mkAct->ArrowTo.X = mkAct->LeftTop.X + mkAct->Width / 2;
				mkAct->ArrowTo.Y = mkAct->LeftTop.Y + mkAct->Height / 2;
				//
				mkPre = mkAct;
			}
		}
		_chart->Draw();
		bRedraw = false;
	}
}

//Tips
void DonutChart::ConfigMarksTipTool()
{
	TMarksTipTool *marksTipTool = new TMarksTipTool(_chart);
	marksTipTool->SystemHints = false;
	marksTipTool->OnGetText = OnMouseOverSerie;
	_chart->Tools->Add(marksTipTool);
}

void __fastcall DonutChart::OnMouseOverSerie(TMarksTipTool* pSender, System::UnicodeString& pText)
{
	pText = "Tip :" + pText;
}

std::vector<TColor>DonutChart::InitColorLst()
{
	std::vector<TColor>vectRet;
	vectRet.push_back(clRed);
	vectRet.push_back(clWebDarkOrange);
	vectRet.push_back(clGreen);
	vectRet.push_back(clWebDarkCyan);
	vectRet.push_back(clBlue);
	vectRet.push_back(clPurple);
	return vectRet;
}
